import pystan

def calc_stats( d ):
    data = sorted(d)
    n = len(data)
    p025 = data[int(0.025 * n)]
    med = data[int(0.5 * n)]
    p975 = data[int(0.975 * n)]
    sum = 0.0
    for v in data:
        sum += v
    mean = sum / float(n)

    n_neg = 0
    while data[n_neg] < 0.0:
        n_neg += 1
        if n_neg >= n:
            break
    prop_neg = n_neg / n
    n_pos = 0
    while data[n - 1 - n_pos] > 0.0:
        n_pos += 1
        if n_pos >= n:
            break
    prop_pos = n_pos / n
    
    return mean, med, p025, p975, prop_neg, prop_pos



#
#       Prepare the input data file
#

fn_in = input("Input data file = ")

f_in = open(fn_in, "r")

#
#       Set the contents of the input data file in the object data_f
#

data_f = f_in.readlines()
f_in.close()

#
#       Prepare the output file
#

fn_out = input("Output data file = ")
f_out = open(fn_out, "w")
f_out.write("Input data file = " + fn_in + "\n\n")

pos = 0
while True:
    if len(data_f[pos]) > 0:
        if data_f[pos][0] == "/":
            break
    pos += 1

pos += 1
p = int(data_f[pos])
pos += 1
var_name = data_f[pos].split()
print('\nlen(var_name) = ', len(var_name))
print('\np = ', p, '\nVar_Names = ', var_name)
f_out.write('\np = {0}\nVar_Names = {1}'.format(p, var_name))

N = 0
Y = []
X = []
pos += 1
while not(data_f[pos][0] == '/'):
    temp_str = data_f[pos].split()
    Y.append([])
    Y[N].append(int(temp_str[0]))
    Y[N].append(int(temp_str[1]))
    X.append([])
    X[N].append(1)
    for i in range(p):
        X[N].append(float(temp_str[i+2]))
    N += 1
    pos += 1
    
f_out.write('\nData...\n')
for i in range(N):
    f_out.write('{0:>5}:   {1:<7}   {2:<7}   '.format(i+1, Y[i][0], Y[i][1]))
    for j in range(p):
        f_out.write('{0:>10}'.format(X[i][j+1]))
    f_out.write('\n')
f_out.write('\n')

Data = {'N': N, 'p': (p + 1), 'Y': Y, 'X': X}

fit = pystan.stan(file = 'LgstcRegGrp.stan', data = Data, seed = 999, n_jobs = 1)

print(fit)

b_smpls = fit['b']

n_smpls = len(b_smpls)
param_b = []
for i in range(p+1):
    param_b.append([])

for i in range(n_smpls):
    for j in range(p+1):
        param_b[j].append(b_smpls[i][j])

mean, med, p025, p975, prop_neg, prop_pos = calc_stats(param_b[0])
print('\nb[0]<const>:\n     {0:>.5}(mean)  {1:>.5}(med)  95%CI [{2:>.5}, {3:>.5}]'.
      format(mean, med, p025, p975))
print('     Prop(v <= 0.0) = {0:<.4f}     Prop(v >= 0.0) = {1:<.4f}'.format(prop_neg, prop_pos))
f_out.write('\nb[0]<const>:\n     {0:>.5}(mean)  {1:>.5}(med)  95%CI [{2:>.5}, {3:>.5}]\n'.
      format(mean, med, p025, p975))
f_out.write('     Prop(v <= 0.0) = {0:<.4f}     Prop(v >= 0.0) = {1:<.4f}\n'.
            format(prop_neg, prop_pos))

for i in range(p):
    print('\nb[{0}]<{1}>:'.format(i+1, var_name[i+2]))
    f_out.write('\nb[{0}]<{1}>:'.format(i+1, var_name[i+2]))
    mean, med, p025, p975, prop_neg, prop_pos = calc_stats(param_b[i+1])
    print('     {0:>.5}(mean)  {1:>.5}(med)  95%CI [{2:>.5}, {3:>.5}]'.
          format(mean, med, p025, p975))
    print('     Prop(v <= 0.0) = {0:<.4f}     Prop(v >= 0.0) = {1:<.4f}'.format(prop_neg, prop_pos))
    f_out.write('\n     {0:>.5}(mean)  {1:>.5}(med)  95%CI [{2:>.5}, {3:>.5}]\n'.
          format(mean, med, p025, p975))
    f_out.write('     Prop(v <= 0.0) = {0:<.4f}     Prop(v >= 0.0) = {1:<.4f}\n'.
            format(prop_neg, prop_pos))
f_out.write('\n')
    
while True:
    pos += 1
    while True:
        if len(data_f[pos]) > 0:
            if data_f[pos][0] == '/':
                break
        pos += 1

    pos += 1
    ck = int(data_f[pos])
    if ck < 0:
        break
    pos += 1
    s_cntrst = data_f[pos].split()
    cntrst = []
    for i in range(p):
        cntrst.append(float(s_cntrst[i]))
    print('\ncontrast = ', end = '')
    for i in range(p):
        print('  {0:<.5}<{1}>'.format(cntrst[i], var_name[i+2]), end = '')
    print(' ')
    f_out.write('\ncontrast = ')
    for i in range(p):
        f_out.write('    {0:<.5}<{1}>'.format(cntrst[i], var_name[i+2]))
    f_out.write('\n')

    c_b = []
    for i in range(n_smpls):
        v = 0.0
        for j in range(p):
            v += cntrst[j] * param_b[j+1][i]
        c_b.append(v)
    mean, med, p025, p975, prop_neg, prop_pos = calc_stats(c_b)
    print('     {0:>.5}(mean)  {1:>.5}(med)  95%CI [{2:>.5}, {3:>.5}]'.
                format(mean, med, p025, p975))
    print('     Prop(v <= 0.0) = {0:<.4f}     Prop(v >= 0.0) = {1:<.4f}'.format(prop_neg, prop_pos))
    f_out.write('     {0:>.5}(mean)  {1:>.5}(med)  95%CI [{2:>.5}, {3:>.5}]\n'.
                format(mean, med, p025, p975))
    f_out.write('     Prop(v <= 0.0) = {0:<.4f}     Prop(v >= 0.0) = {1:<.4f}\n'.
            format(prop_neg, prop_pos))
        

f_out.close()
print('\n')
print(fn_out, ' was saved.')
